using Microsoft.Extensions.Configuration;
using System.Text.Json;
using System.Text.Json.Serialization;
using View_by_Distance.Shared.Models;

namespace View_by_Distance.Compare.Models;

public record AppSettings(ResultSettings ResultSettings,
                          MetadataSettings MetadataSettings,
                          PeopleSettings PeopleSettings,
                          DistanceSettings DistanceSettings,
                          CompareSettings CompareSettings)
{

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, AppSettingsSourceGenerationContext.Default.AppSettings);
        return result;
    }

    private static void Verify(AppSettings appSettings)
    {
        if (appSettings.DistanceSettings.RangeDaysDeltaTolerance.Length != 3)
            throw new NullReferenceException(nameof(appSettings.DistanceSettings.RangeDaysDeltaTolerance));
        if (appSettings.DistanceSettings.RangeDistanceTolerance.Length != 3)
            throw new NullReferenceException(nameof(appSettings.DistanceSettings.RangeDistanceTolerance));
        if (appSettings.DistanceSettings.RangeFaceAreaTolerance.Length != 3)
            throw new NullReferenceException(nameof(appSettings.DistanceSettings.RangeFaceAreaTolerance));
        if (appSettings.DistanceSettings.RangeFaceConfidence.Length != 3)
            throw new NullReferenceException(nameof(appSettings.DistanceSettings.RangeFaceConfidence));
        _ = DateTime.Now.AddDays(-appSettings.DistanceSettings.RangeDaysDeltaTolerance[1]);
        if (appSettings.DistanceSettings.SaveSortingWithoutPerson && appSettings.PeopleSettings.JLinks.Length > 0)
            throw new Exception("Settings has SaveSortingWithoutPerson and JLinks!");
        if (appSettings.DistanceSettings.SaveSortingWithoutPerson && !string.IsNullOrEmpty(appSettings.DistanceSettings.FocusModel))
            throw new Exception("Settings has SaveSortingWithoutPerson and FocusModel!");
        if (appSettings.DistanceSettings.SaveSortingWithoutPerson && !string.IsNullOrEmpty(appSettings.DistanceSettings.FocusDirectory))
            throw new Exception("Settings has SaveSortingWithoutPerson and FocusDirectory!");
        if (appSettings.CompareSettings.MaxDegreeOfParallelism > Environment.ProcessorCount)
            throw new Exception("MaxDegreeOfParallelism must be =< Environment.ProcessorCount!");
        if (!string.IsNullOrEmpty(appSettings.DistanceSettings.FocusDirectory) && appSettings.DistanceSettings.FocusDirectory.Length != 2)
            throw new NotSupportedException($"{nameof(appSettings.DistanceSettings.FocusDirectory)} currently only works with output directory! Example 00.");
    }

    public static AppSettings Get(IConfigurationRoot configurationRoot)
    {
        AppSettings result;
#pragma warning disable IL3050, IL2026
        ResultSettings? resultSettings = configurationRoot.GetSection(nameof(ResultSettings)).Get<ResultSettings>();
        MetadataSettings? metadataSettings = configurationRoot.GetSection(nameof(MetadataSettings)).Get<MetadataSettings>();
        PeopleSettings? peopleSettings = configurationRoot.GetSection(nameof(PeopleSettings)).Get<PeopleSettings>();
        DistanceSettings? distanceSettings = configurationRoot.GetSection(nameof(DistanceSettings)).Get<DistanceSettings>();
        CompareSettings? compareSettings = configurationRoot.GetSection(nameof(CompareSettings)).Get<CompareSettings>();
#pragma warning restore IL3050, IL2026
        if (resultSettings is null || metadataSettings is null || peopleSettings is null || distanceSettings is null || compareSettings?.Company is null)
        {
            List<string> paths = [];
            foreach (IConfigurationProvider configurationProvider in configurationRoot.Providers)
            {
                if (configurationProvider is not Microsoft.Extensions.Configuration.Json.JsonConfigurationProvider jsonConfigurationProvider)
                    continue;
                if (jsonConfigurationProvider.Source.FileProvider is not Microsoft.Extensions.FileProviders.PhysicalFileProvider physicalFileProvider)
                    continue;
                paths.Add(physicalFileProvider.Root);
            }
            throw new NotSupportedException($"Not found!{Environment.NewLine}{string.Join(Environment.NewLine, paths.Distinct())}");
        }
        result = new(resultSettings, metadataSettings, peopleSettings, distanceSettings, compareSettings);
        Verify(result);
        return result;
    }

}

[JsonSourceGenerationOptions(WriteIndented = true)]
[JsonSerializable(typeof(AppSettings))]
internal partial class AppSettingsSourceGenerationContext : JsonSerializerContext
{
}